@@ -2,7 +2,7 @@
  * The SIP library code that implements the interface to the optional module
  * supplied Qt support.
  *
- * Copyright (c) 2006
+ * Copyright (c) 2007
  * 	Riverbank Computing Limited <info@riverbankcomputing.co.uk>
  * 
  * This file is part of SIP.
@@ -32,14 +32,13 @@ static PyObject *py_sender = NULL;  /* The last Python signal sender. */
 
 static int isSameSlot(sipSlot *,PyObject *,const char *);
 static int emitQtSig(sipWrapper *,const char *,PyObject *);
-static int emitToSlotList(sipPySigRx *,PyObject *);
+static int emitToSlotList(sipSlotList *rxlist, PyObject *sigargs);
 static int addSlotToPySigList(sipWrapper *,const char *,PyObject *,const char *);
 static void removeSlotFromPySigList(sipWrapper *,const char *,PyObject *,const char *);
 static PyObject *getWeakRef(PyObject *obj);
 static sipPySig *findPySignal(sipWrapper *,const char *);
 static char *sipStrdup(const char *);
 static int saveSlot(sipSlot *sp, PyObject *rxObj, const char *slot);
-static sipSignature *parseSignature(const char *sig);
 static void *createUniversalSlot(sipWrapper *txSelf, const char *sig, PyObject *rxObj, const char *slot, const char **member);
 static void *findSignal(void *txrx, const char **sig);
 static void *newSignal(void *txrx, const char **sig);
@@ -96,7 +95,7 @@ int sip_api_same_connection(sipSlotConnection *conn, void *tx, const char *sig,
 /*
  * Parse the signal arguments for a connection.
  */
-static sipSignature *parseSignature(const char *sig)
+sipSignature *sip_api_parse_signature(const char *sig)
 {
     static sipSignature *psig_list = NULL;
     sipSignature *psig;
@@ -269,6 +268,11 @@ static sipSignature *parseSignature(const char *sig)
                 case 7:
                     if (strncmp(dp, "__int64", 7) == 0)
                         sat = longlong_sat;
+                    else if (strncmp(dp, "wchar_t", 7) == 0)
+                    {
+                        sat = (indir ? wstring_sat : wchar_sat);
+                        unsup = (isref || indir > 1);
+                    }
                     break;
 
                 case 8:
@@ -287,11 +291,6 @@ static sipSignature *parseSignature(const char *sig)
                             unsup = FALSE;
                         }
                     }
-                    else if (strncmp(dp, "PyObject", 8) == 0 && indir == 1)
-                    {
-                        sat = pyobject_sat;
-                        unsup = FALSE;
-                    }
                     break;
 
                 case 9:
@@ -320,6 +319,11 @@ static sipSignature *parseSignature(const char *sig)
                         sat = (indir ? ustring_sat : uchar_sat);
                         unsup = (isref || indir > 1);
                     }
+                    else if (strncmp(dp, "PyQt_PyObject", 13) == 0 && indir == 0)
+                    {
+                        sat = pyobject_sat;
+                        unsup = FALSE;
+                    }
                     break;
 
                 case 14:
@@ -387,7 +391,7 @@ static void *findSignal(void *txrx, const char **sig)
     if (sipQtSupport->qt_is_qt_signal(txrx, *sig))
         return txrx;
 
-    if ((psig = parseSignature(*sig)) == NULL)
+    if ((psig = sip_api_parse_signature(*sig)) == NULL)
         return NULL;
 
     /* Find an ordinary universal signal. */
@@ -417,7 +421,7 @@ static void *newSignal(void *txrx, const char **sig)
     if (sipQtSupport->qt_is_qt_signal(txrx, *sig))
         return txrx;
 
-    if ((psig = parseSignature(*sig)) == NULL)
+    if ((psig = sip_api_parse_signature(*sig)) == NULL)
         return NULL;
 
     /* Create an ordinary universal signal. */
@@ -444,7 +448,7 @@ static void *createUniversalSlot(sipWrapper *txSelf, const char *sig,
         return 0;
 
     /* Parse the signature and create the universal slot. */
-    if ((conn.sc_signature = parseSignature(sig)) == NULL || (us = sipQtSupport->qt_create_universal_slot(txSelf, &conn, member)) == NULL)
+    if ((conn.sc_signature = sip_api_parse_signature(sig)) == NULL || (us = sipQtSupport->qt_create_universal_slot(txSelf, &conn, member)) == NULL)
     {
         sip_api_free_connection(&conn);
         return 0;
@@ -482,7 +486,7 @@ int sip_api_emit_signal(PyObject *self,const char *sig,PyObject *sigargs)
         if (strchr(sig, '(') == NULL)
             return sipQtSupport->qt_emit_signal_shortcut(tx, sig, sigargs);
 
-        if ((psig = parseSignature(sig)) == NULL)
+        if ((psig = sip_api_parse_signature(sig)) == NULL)
             return -1;
 
         if (psig->sg_nrargs != PyTuple_GET_SIZE(sigargs))
@@ -580,7 +584,16 @@ int sip_api_emit_to_slot(sipSlot *slot, PyObject *sigargs)
         return sip_api_emit_signal(slot -> pyobj,slot -> name,sigargs);
 
     /* Get the object to call, resolving any weak references. */
-    if (slot -> weakSlot == NULL)
+    if (slot->weakSlot == Py_True)
+    {
+        /*
+         * The slot is guaranteed to be Ok because it has an extra reference or
+         * is None.
+         */
+        sref = slot->pyobj;
+        Py_INCREF(sref);
+    }
+    else if (slot -> weakSlot == NULL)
         sref = NULL;
     else if ((sref = PyWeakref_GetObject(slot -> weakSlot)) == NULL)
         return -1;
@@ -771,7 +784,7 @@ int sip_api_emit_to_slot(sipSlot *slot, PyObject *sigargs)
 /*
  * Send a signal to the slots (Qt or Python) in a Python list.
  */
-static int emitToSlotList(sipPySigRx *rxlist,PyObject *sigargs)
+static int emitToSlotList(sipSlotList *rxlist,PyObject *sigargs)
 {
     int rc;
 
@@ -780,7 +793,7 @@ static int emitToSlotList(sipPySigRx *rxlist,PyObject *sigargs)
 
     while (rxlist != NULL && rc >= 0)
     {
-        sipPySigRx *next;
+        sipSlotList *next;
 
         /*
          * We get the next in the list before calling the slot in case the list
@@ -805,7 +818,7 @@ static int addSlotToPySigList(sipWrapper *txSelf,const char *sig,
                   PyObject *rxObj,const char *slot)
 {
     sipPySig *ps;
-    sipPySigRx *psrx;
+    sipSlotList *psrx;
 
     /* Create a new one if necessary. */
     if ((ps = findPySignal(txSelf,sig)) == NULL)
@@ -826,7 +839,7 @@ static int addSlotToPySigList(sipWrapper *txSelf,const char *sig,
     }
 
     /* Create the new receiver. */
-    if ((psrx = (sipPySigRx *)sip_api_malloc(sizeof (sipPySigRx))) == NULL)
+    if ((psrx = (sipSlotList *)sip_api_malloc(sizeof (sipSlotList))) == NULL)
         return -1;
 
     if (saveSlot(&psrx->rx, rxObj, slot) < 0)
@@ -845,30 +858,40 @@ static int addSlotToPySigList(sipWrapper *txSelf,const char *sig,
 /*
  * Compare two slots to see if they are the same.
  */
-static int isSameSlot(sipSlot *slot1,PyObject *rxobj2,const char *slot2)
+static int isSameSlot(sipSlot *slot1, PyObject *rxobj2, const char *slot2)
 {
     /* See if they are signals or Qt slots, ie. they have a name. */
-    if (slot1 -> name != NULL)
-        return (slot2 != NULL &&
-            sipQtSupport->qt_same_name(slot1 -> name,slot2) &&
-            slot1 -> pyobj == rxobj2);
-
-    /* Both must be Python slots. */
     if (slot2 != NULL)
-        return 0;
+    {
+        if (slot1->name == NULL || slot1->name[0] == '\0')
+            return 0;
 
-    /* See if they are Python methods. */
-    if (slot1 -> pyobj == NULL)
-        return (PyMethod_Check(rxobj2) &&
-            slot1 -> meth.mfunc == PyMethod_GET_FUNCTION(rxobj2) &&
-            slot1 -> meth.mself == PyMethod_GET_SELF(rxobj2) &&
-            slot1 -> meth.mclass == PyMethod_GET_CLASS(rxobj2));
+        return (sipQtSupport->qt_same_name(slot1->name, slot2) && slot1->pyobj == rxobj2);
+    }
 
+    /* See if they are pure Python methods. */
     if (PyMethod_Check(rxobj2))
-        return 0;
+    {
+        if (slot1->pyobj != NULL)
+            return 0;
+
+        return (slot1->meth.mfunc == PyMethod_GET_FUNCTION(rxobj2) &&
+                slot1->meth.mself == PyMethod_GET_SELF(rxobj2) &&
+                slot1->meth.mclass == PyMethod_GET_CLASS(rxobj2));
+    }
+
+    /* See if they are wrapped C++ methods. */
+    if (PyCFunction_Check(rxobj2))
+    {
+        if (slot1->name == NULL || slot1->name[0] != '\0')
+            return 0;
+
+        return (slot1->pyobj == PyCFunction_GET_SELF(rxobj2) &&
+                strcmp(&slot1->name[1], ((PyCFunctionObject *)rxobj2)->m_ml->ml_name) == 0);
+    }
 
     /* The objects must be the same. */
-    return (slot1 -> pyobj == rxobj2);
+    return (slot1->pyobj == rxobj2);
 }
 
 
@@ -903,7 +926,7 @@ void *sipGetRx(sipWrapper *txSelf,const char *sigargs,PyObject *rxObj,
 
 
 /*
- * Convert a Python receiver (either a Python signal or slot or a Qt signals or
+ * Convert a Python receiver (either a Python signal or slot or a Qt signal or
  * slot) to a Qt receiver.  It is only ever called when the signal is a Qt
  * signal.  Return NULL is there was an error.
  */
@@ -960,9 +983,7 @@ PyObject *sip_api_connect_rx(PyObject *txObj,const char *sig,PyObject *rxObj,
         if ((rx = sip_api_convert_rx(txSelf, sig, rxObj, slot, &member)) == NULL)
             return NULL;
 
-        Py_BEGIN_ALLOW_THREADS
         res = sipQtSupport->qt_connect(tx, real_sig, rx, member, type);
-        Py_END_ALLOW_THREADS
 
         return PyBool_FromLong(res);
     }
@@ -989,7 +1010,7 @@ PyObject *sip_api_disconnect_rx(PyObject *txObj,const char *sig,
     {
         void *tx, *rx;
         const char *member;
-        PyObject *res;
+        int res;
 
         if ((tx = sip_api_get_cpp_ptr(txSelf, sipQObjectClass)) == NULL)
             return NULL;
@@ -1003,7 +1024,7 @@ PyObject *sip_api_disconnect_rx(PyObject *txObj,const char *sig,
         /* Handle Python signals. */
         tx = findSignal(tx, &sig);
 
-        res = PyBool_FromLong(sipQtSupport->qt_disconnect(tx, sig, rx, member));
+        res = sipQtSupport->qt_disconnect(tx, sig, rx, member);
 
         /*
          * Delete it if it is a universal slot as this will be it's only
@@ -1012,7 +1033,7 @@ PyObject *sip_api_disconnect_rx(PyObject *txObj,const char *sig,
          */
         sipQtSupport->qt_destroy_universal_slot(rx);
 
-        return res;
+        return PyBool_FromLong(res);
     }
 
     /* Handle Python signals. */
@@ -1033,16 +1054,16 @@ static void removeSlotFromPySigList(sipWrapper *txSelf,const char *sig,
 
     if ((ps = findPySignal(txSelf,sig)) != NULL)
     {
-        sipPySigRx **psrxp;
+        sipSlotList **psrxp;
 
         for (psrxp = &ps -> rxlist; *psrxp != NULL; psrxp = &(*psrxp) -> next)
         {
-            sipPySigRx *psrx = *psrxp;
+            sipSlotList *psrx = *psrxp;
 
             if (isSameSlot(&psrx -> rx,rxObj,slot))
             {
                 *psrxp = psrx -> next;
-                sipFreePySigRx(psrx);
+                sipFreeSlotList(psrx);
                 break;
             }
         }
@@ -1057,6 +1078,8 @@ static void freeSlot(sipSlot *slot)
 {
     if (slot->name != NULL)
         sip_api_free(slot->name);
+    else if (slot->weakSlot == Py_True)
+        Py_DECREF(slot->pyobj);
 
     /* Remove any weak reference. */
     Py_XDECREF(slot->weakSlot);
@@ -1064,9 +1087,9 @@ static void freeSlot(sipSlot *slot)
 
 
 /*
- * Free a sipPySigRx structure on the heap.
+ * Free a sipSlotList structure on the heap.
  */
-void sipFreePySigRx(sipPySigRx *rx)
+void sipFreeSlotList(sipSlotList *rx)
 {
     freeSlot(&rx->rx);
     sip_api_free(rx);
@@ -1138,7 +1161,7 @@ static int saveSlot(sipSlot *sp, PyObject *rxObj, const char *slot)
                  * at "obj, SLOT('meth()')" rather than "obj.meth" (see below).
                  */
 
-                char *meth;
+                const char *meth;
 
                 /* Get the method name. */
                 meth = ((PyCFunctionObject *)rxObj) -> m_ml -> ml_name;
@@ -1159,13 +1182,14 @@ static int saveSlot(sipSlot *sp, PyObject *rxObj, const char *slot)
             else
             {
                 /*
-                 * It's unlikely that we will succeed in getting a weak
-                 * reference to the slot, but there is no harm in trying (and
-                 * future versions of Python may support references to more
-                 * object types).
+                 * Give the slot an extra reference to keep it alive and
+                 * remember we have done so by treating weakSlot specially.
                  */
-                sp -> pyobj = rxObj;
-                sp -> weakSlot = getWeakRef(rxObj);
+                Py_INCREF(rxObj);
+                sp->pyobj = rxObj;
+
+                Py_INCREF(Py_True);
+                sp->weakSlot = Py_True;
             }
         }
     }
