@@ -1,7 +1,7 @@
 /*
  * This file defines the SIP library internal interfaces.
  *
- * Copyright (c) 2006
+ * Copyright (c) 2007
  * 	Riverbank Computing Limited <info@riverbankcomputing.co.uk>
  * 
  * This file is part of SIP.
@@ -15,18 +15,18 @@
 
 
 #ifndef _SIPINT_H
-#define	_SIPINT_H
+#define _SIPINT_H
 
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-#undef	TRUE
-#define	TRUE		1
+#undef  TRUE
+#define TRUE        1
 
-#undef	FALSE
-#define	FALSE		0
+#undef  FALSE
+#define FALSE       0
 
 
 /*
@@ -34,8 +34,8 @@ extern "C" {
  */
 typedef struct
 {
-	void *key;			/* The C/C++ address. */
-	sipWrapper *first;		/* The first object at this address. */
+    void *key;                  /* The C/C++ address. */
+    sipWrapper *first;          /* The first object at this address. */
 } sipHashEntry;
 
 
@@ -45,31 +45,31 @@ typedef struct
  */
 typedef struct
 {
-	int primeIdx;			/* Index into table sizes. */
-	unsigned long size;		/* Size of hash table. */
-	unsigned long unused;		/* Nr. unused in hash table. */
-	unsigned long stale;		/* Nr. stale in hash table. */
-	sipHashEntry *hash_array;	/* Current hash table. */
+    int primeIdx;               /* Index into table sizes. */
+    unsigned long size;         /* Size of hash table. */
+    unsigned long unused;       /* Nr. unused in hash table. */
+    unsigned long stale;        /* Nr. stale in hash table. */
+    sipHashEntry *hash_array;   /* Current hash table. */
 } sipObjectMap;
 
 
-extern PyInterpreterState *sipInterpreter;	/* The interpreter. */
+extern PyInterpreterState *sipInterpreter;  /* The interpreter. */
 
 
-extern sipQtAPI *sipQtSupport;		/* The Qt support API. */
-extern sipWrapperType *sipQObjectClass;	/* The Python QObject class. */
+extern sipQtAPI *sipQtSupport;  /* The Qt support API. */
+extern sipWrapperType *sipQObjectClass; /* The Python QObject class. */
 
 void *sip_api_convert_rx(sipWrapper *txSelf, const char *sigargs,
-			 PyObject *rxObj, const char *slot,
-			 const char **memberp);
-void *sipGetRx(sipWrapper *txSelf,const char *sigargs,PyObject *rxObj,
-	       const char *slot,const char **memberp);
-int sip_api_emit_signal(PyObject *self,const char *sig,PyObject *sigargs);
+        PyObject *rxObj, const char *slot, const char **memberp);
+void *sipGetRx(sipWrapper *txSelf, const char *sigargs, PyObject *rxObj,
+        const char *slot, const char **memberp);
+int sip_api_emit_signal(PyObject *self, const char *sig, PyObject *sigargs);
 PyObject *sip_api_get_sender();
-PyObject *sip_api_connect_rx(PyObject *txObj,const char *sig,
-			     PyObject *rxObj,const char *slot, int type);
-PyObject *sip_api_disconnect_rx(PyObject *txObj,const char *sig,
-				PyObject *rxObj,const char *slot);
+PyObject *sip_api_connect_rx(PyObject *txObj, const char *sig, PyObject *rxObj,
+        const char *slot, int type);
+PyObject *sip_api_disconnect_rx(PyObject *txObj, const char *sig,
+        PyObject *rxObj,const char *slot);
+sipSignature *sip_api_parse_signature(const char *sig);
 
 
 /*
@@ -79,7 +79,8 @@ void *sip_api_malloc(size_t nbytes);
 void sip_api_free(void *mem);
 void *sip_api_get_cpp_ptr(sipWrapper *w,sipWrapperType *type);
 PyObject *sip_api_convert_from_instance(void *cppPtr, sipWrapperType *type,
-					PyObject *transferObj);
+        PyObject *transferObj);
+void sip_api_common_dtor(sipWrapper *sipSelf);
 void sip_api_start_thread(void);
 void sip_api_end_thread(void);
 PyObject *sip_api_convert_from_void_ptr(void *val);
@@ -88,17 +89,17 @@ int sip_api_wrapper_check(PyObject *o);
 void sip_api_free_connection(sipSlotConnection *conn);
 int sip_api_emit_to_slot(sipSlot *slot, PyObject *sigargs);
 int sip_api_same_connection(sipSlotConnection *conn, void *tx, const char *sig,
-			    PyObject *rxObj, const char *slot);
+        PyObject *rxObj, const char *slot);
 
 
 /*
  * These are not part of the SIP API but are used within the SIP module.
  */
-void sipFreePySigRx(sipPySigRx *rx);
+void sipFreeSlotList(sipSlotList *rx);
 void sipSaveMethod(sipPyMethod *pm,PyObject *meth);
 void *sipGetPending(sipWrapper **op, int *fp);
 PyObject *sipWrapSimpleInstance(void *cppPtr, sipWrapperType *type,
-		sipWrapper *owner, int initflags);
+        sipWrapper *owner, int initflags);
 
 void sipOMInit(sipObjectMap *om);
 void sipOMFinalise(sipObjectMap *om);
