@@ -2,7 +2,7 @@
 # extension modules created with SIP.  It provides information about file
 # locations, version numbers etc., and provides some classes and functions.
 #
-# Copyright (c) 2006
+# Copyright (c) 2007
 # 	Riverbank Computing Limited <info@riverbankcomputing.co.uk>
 # 
 # This file is part of SIP.
@@ -19,6 +19,7 @@ import os
 import string
 import types
 import stat
+import re
 
 
 # These are installation specific values created when SIP was configured.
@@ -110,6 +111,13 @@ class _UniqueList:
         if value not in self._list:
             self._list.append(value)
 
+    def lextend(self, value):
+        """A normal list extend ignoring the uniqueness.
+
+        value is the list of elements to append.
+        """
+        self._list.extend(value)
+
     def extend(self, value):
         """Append each element of a value to a list if it isn't already
         present.
@@ -180,7 +188,7 @@ class Makefile:
     """
     def __init__(self, configuration, console=0, qt=0, opengl=0, python=0,
                  threaded=0, warnings=1, debug=0, dir=None,
-                 makefile="Makefile", installs=None):
+                 makefile="Makefile", installs=None, universal=''):
         """Initialise an instance of the target.  All the macros are left
         unchanged allowing scripts to manipulate them at will.
 
@@ -202,6 +210,8 @@ class Makefile:
         part list, the first of which is the source and the second is the
         destination.  If the source is another list then it is a set of source
         files and the destination is a directory.
+        universal is the name of the SDK if the target is a MacOS/X universal
+        binary.
         """
         if qt:
             if not hasattr(configuration, "qt_version"):
@@ -219,6 +229,9 @@ class Makefile:
         else:
             self._threaded = threaded
 
+        if sys.platform != "darwin":
+            universal = ''
+
         self.config = configuration
         self.console = console
         self._qt = qt
@@ -229,6 +242,7 @@ class Makefile:
         self._dir = dir
         self._makefile = makefile
         self._installs = installs
+        self._universal = universal
 
         self._finalised = 0
 
@@ -245,7 +259,7 @@ class Makefile:
             # These require special handling as they are (potentially) a set of
             # space separated values rather than a single value that might
             # contain spaces.
-            if m == "DEFINES" or m[:6] in ("INCDIR", "LIBDIR"):
+            if m in ("DEFINES", "CONFIG") or m[:6] in ("INCDIR", "LIBDIR"):
                 val = string.split(val)
 
             # We also want to treat lists of libraries in the same way so that
@@ -291,6 +305,11 @@ class Makefile:
         to a Makefile.
         """
         # Extract the things we might need from the Windows Qt configuration.
+        # Note that we used to think that if Qt was built with exceptions, RTTI
+        # and STL support enabled then anything that linked against it also
+        # needed the same flags.  However, detecting this was broken for some
+        # time and nobody complained.  For the moment we'll leave the code in
+        # but it will never be used.
         if self._qt:
             wcfg = string.split(self.config.qt_winconfig)
             win_shared = ("shared" in wcfg)
@@ -328,6 +347,15 @@ class Makefile:
         libdir.extend(self.extra_lib_dirs)
         libdir.extend(self.optional_list("LIBDIR"))
 
+        # Handle MacOS/X universal binaries.
+        if self._universal:
+            unicflags = ('-arch ppc -arch i386 -isysroot %s' % self._universal).split()
+            unilflags = ('-arch ppc -arch i386 -Wl,-syslibroot,%s' % self._universal).split()
+
+            cflags.lextend(unicflags)
+            cxxflags.lextend(unicflags)
+            lflags.lextend(unilflags)
+
         # Don't use a unique list as libraries may need to be searched more
         # than once.  Also MacOS/X uses the form "-framework lib" so we don't
         # want to lose the multiple "-framework".
@@ -481,8 +509,12 @@ class Makefile:
                         defines.append("QT_NETWORK_LIB")
                     elif mod == "QtOpenGL":
                         defines.append("QT_OPENGL_LIB")
+                    elif mod == "QtScript":
+                        defines.append("QT_SCRIPT_LIB")
                     elif mod == "QtSql":
                         defines.append("QT_SQL_LIB")
+                    elif mod == "QtTest":
+                        defines.append("QT_TEST_LIB")
                     elif mod == "QtXml":
                         defines.append("QT_XML_LIB")
             elif self._threaded:
@@ -509,16 +541,32 @@ class Makefile:
                     "QtGui":        ("QtCore", ),
                     "QtNetwork":    ("QtCore", ),
                     "QtOpenGL":     ("QtCore", "QtGui"),
+                    "QtScript":     ("QtCore", ),
                     "QtSql":        ("QtCore", ),
                     "QtSvg":        ("QtCore", "QtGui", "QtXml"),
-                    "QtXml":        ("QtCore", )
+                    "QtTest":       ("QtCore", "QtGui"),
+                    "QtXml":        ("QtCore", ),
+                    "QtDesigner":   ("QtCore", "QtGui"),
+                    "QAxContainer": ("QtCore", "QtGui")
                 }
 
                 # The QtSql .prl file doesn't include QtGui as a dependency (at
                 # least on Linux) so we explcitly set the dependency here for
                 # everything.
-                if "QtSql" in self._qt and "QtGui" not in self._qt:
-                    self._qt.append("QtGui")
+                if "QtSql" in self._qt:
+                    if "QtGui" not in self._qt:
+                        self._qt.append("QtGui")
+
+                # With Qt v4.2.0, the QtAssistantClient library is now a shared
+                # library on UNIX. The QtAssistantClient .prl file doesn't
+                # include QtGui and QtNetwork as a dependency any longer.  This
+                # seems to be a bug in Qt v4.2.0.  We explicitly set the
+                # dependencies here.
+                if self.config.qt_version >= 0x040200 and "QtAssistant" in self._qt:
+                    if "QtGui" not in self._qt:
+                        self._qt.append("QtGui")
+                    if "QtNetwork" not in self._qt:
+                        self._qt.append("QtNetwork")
 
                 for mod in self._qt:
                     lib = self._qt4_module_to_lib(mod)
@@ -546,7 +594,7 @@ class Makefile:
 
                         libs.extend(deps.as_list())
                     else:
-                        libs.extend(self._dependent_libs(lib))
+                        libs.extend(self._dependent_libs(lib, self._is_framework(mod)))
             else:
                 # Windows needs the version number appended if Qt is a DLL.
                 qt_lib = self.config.qt_lib
@@ -576,15 +624,21 @@ class Makefile:
             qtincdir = self.optional_list("INCDIR_QT")
 
             if qtincdir:
-                incdir.extend(qtincdir)
-
                 if self.config.qt_version >= 0x040000:
                     for mod in self._qt:
-                        if self._is_framework(mod):
+                        if mod == "QAxContainer":
+                            incdir.append(os.path.join(qtincdir[0], "ActiveQt"))
+                        elif self._is_framework(mod):
+                            if mod == "QtAssistant" and self.config.qt_version < 0x040202:
+                                mod = "QtAssistantClient"
+
                             incdir.append(os.path.join(libdir_qt[0], mod + ".framework", "Headers"))
                         else:
                             incdir.append(os.path.join(qtincdir[0], mod))
 
+                # This must go after the module include directories.
+                incdir.extend(qtincdir)
+
         if self._opengl:
             incdir.extend(self.optional_list("INCDIR_OPENGL"))
             lflags.extend(self.optional_list("LFLAGS_OPENGL"))
@@ -624,7 +678,7 @@ class Makefile:
     def _is_framework(self, mod):
         """Return true if the given Qt module is a framework.
         """
-        return (self.config.qt_framework and mod != "QtAssistant")
+        return (self.config.qt_framework and (self.config.qt_version >= 0x040200 or mod != "QtAssistant"))
 
     def _qt4_module_to_lib(self, mname):
         """Return the name of the Qt4 library corresponding to a module.
@@ -632,18 +686,25 @@ class Makefile:
         mname is the name of the module.
         """
         if mname == "QtAssistant":
-            lib = "QtAssistantClient"
+            if self.config.qt_version >= 0x040202 and sys.platform == "darwin":
+                lib = mname
+            else:
+                lib = "QtAssistantClient"
         else:
             lib = mname
 
         if self._debug:
             if sys.platform == "win32":
                 lib = lib + "d"
-            else:
+            elif self.config.qt_version < 0x040200 or sys.platform == "darwin":
                 lib = lib + "_debug"
 
-        if sys.platform == "win32" and mname in ("QtCore", "QtGui", "QtNetwork", "QtOpenGL", "QtSql", "QtSvg", "QtXml"):
-            lib = lib + "4"
+        if sys.platform == "win32" and "shared" in string.split(self.config.qt_winconfig):
+            if (mname in ("QtCore", "QtGui", "QtNetwork", "QtOpenGL",
+                          "QtScript", "QtSql", "QtSvg", "QtTest", "QtXml",
+                          "QtDesigner") or
+                (self.config.qt_version >= 0x040200 and mname == "QtAssistant")):
+                lib = lib + "4"
 
         return lib
 
@@ -708,16 +769,19 @@ class Makefile:
 
         return plib
 
-    def _dependent_libs(self, clib):
+    def _dependent_libs(self, clib, framework=0):
         """Return a list of additional libraries (in platform specific form)
         that must be linked with a library.
 
         clib is the library name in cannonical form.
+        framework is set of the library is implemented as a MacOS framework.
         """
         prl_libs = []
 
         if self.generator in ("MSVC", "MSVC.NET", "BMAKE"):
             prl_name = os.path.join(self.config.qt_lib_dir, clib + ".prl")
+        elif sys.platform == "darwin" and framework:
+            prl_name = os.path.join(self.config.qt_lib_dir, clib + ".framework", clib + ".prl")
         else:
             prl_name = os.path.join(self.config.qt_lib_dir, "lib" + clib + ".prl")
 
@@ -1184,7 +1248,7 @@ class ModuleMakefile(Makefile):
     def __init__(self, configuration, build_file, install_dir=None, static=0,
                  console=0, qt=0, opengl=0, threaded=0, warnings=1, debug=0,
                  dir=None, makefile="Makefile", installs=None, strip=1,
-                 export_all=0):
+                 export_all=0, universal=''):
         """Initialise an instance of a module Makefile.
 
         build_file is the file containing the target specific information.  If
@@ -1198,7 +1262,7 @@ class ModuleMakefile(Makefile):
         increases the size of the module and slows down module load times but
         may avoid problems with modules that use exceptions.  The default is 0.
         """
-        Makefile.__init__(self, configuration, console, qt, opengl, 1, threaded, warnings, debug, dir, makefile, installs)
+        Makefile.__init__(self, configuration, console, qt, opengl, 1, threaded, warnings, debug, dir, makefile, installs, universal)
 
         self._build = self.parse_build_file(build_file)
         self._install_dir = install_dir
@@ -1273,7 +1337,10 @@ class ModuleMakefile(Makefile):
             # This allows Apple's Python to be used even if a later python.org
             # version is also installed.
             dl = string.split(sys.exec_prefix, os.sep)
-            dl = dl[:dl.index("Python.framework")]
+            try:
+                dl = dl[:dl.index("Python.framework")]
+            except ValueError:
+                error("SIP requires Python to be built as a framework")
             self.LFLAGS.append("-F%s" % string.join(dl, os.sep))
             self.LFLAGS.append("-framework Python")
 
@@ -1310,6 +1377,11 @@ class ModuleMakefile(Makefile):
                 if self._limit_exports:
                     if sys.platform[:5] == 'linux':
                         self.LFLAGS.extend(['-Wl,--version-script=%s.exp' % self._target])
+                    elif sys.platform[:5] == 'sunos':
+                        if self.required_string('LINK') == 'g++':
+                            self.LFLAGS.extend(['-Wl,-z,noversion', '-Wl,-M,%s.exp' % self._target])
+                        else:
+                            self.LFLAGS.extend(['-z' 'noversion', '-M', '%s.exp' % self._target])
                     elif sys.platform[:5] == 'hp-ux':
                         self.LFLAGS.extend(['-Wl,+e,init%s' % self._target])
                     elif sys.platform[:5] == 'irix' and self.required_string('LINK') != 'g++':
@@ -1360,6 +1432,9 @@ class ModuleMakefile(Makefile):
         if self.static:
             if self.generator in ("MSVC", "MSVC.NET", "BMAKE"):
                 mfile.write("LIB = %s\n" % self.required_string("LIB"))
+            elif self.generator == "MINGW":
+                mfile.write("AR = %s\n" % self.required_string("LIB"))
+                self._ranlib = None
             else:
                 mfile.write("AR = %s\n" % self.required_string("AR"))
 
@@ -1386,6 +1461,9 @@ class ModuleMakefile(Makefile):
                 mfile.write("\t$(LINK) $(LFLAGS) /OUT:$(TARGET) @<<\n")
                 mfile.write("\t  $(OFILES) $(LIBS)\n")
                 mfile.write("<<\n")
+
+                if "embed_manifest_dll" in self.optional_list("CONFIG"):
+                    mfile.write("\tmt -nologo -manifest $(TARGET).manifest -outputresource:$(TARGET);2\n")
         elif self.generator == "BMAKE":
             if self.static:
                 mfile.write("\t-%s $(TARGET)\n" % (self.rm))
@@ -1425,9 +1503,11 @@ class ModuleMakefile(Makefile):
                     mfile.write("\t$(RANLIB) $(TARGET)\n")
             else:
                 if self._limit_exports:
-                    # Create an export file for AIX and Linux.
+                    # Create an export file for AIX, Linux and Solaris.
                     if sys.platform[:5] == 'linux':
                         mfile.write("\t@echo '{ global: init%s; local: *; };' > %s.exp\n" % (self._target, self._target))
+                    elif sys.platform[:5] == 'sunos':
+                        mfile.write("\t@echo '{ global: init%s; local: *; };' > %s.exp\n" % (self._target, self._target))
                     elif sys.platform[:3] == 'aix':
                         mfile.write("\t@echo '#!' >%s.exp" % self._target)
                         mfile.write("; \\\n\t echo 'init%s' >>%s.exp\n" % (self._target, self._target))
@@ -1462,8 +1542,10 @@ class ModuleMakefile(Makefile):
         mfile.write("\nclean:\n")
         self.clean_build_file_objects(mfile, self._build)
 
-        # Remove any export file on AIX and Linux.
-        if self._limit_exports and (sys.platform[:5] == 'linux' or sys.platform[:3] == 'aix'):
+        # Remove any export file on AIX, Linux and Solaris.
+        if self._limit_exports and (sys.platform[:5] == 'linux' or
+                                    sys.platform[:5] == 'sunos' or
+                                    sys.platform[:3] == 'aix'):
             mfile.write("\t-%s %s.exp\n" % (self.rm, self._target))
 
 
@@ -1483,14 +1565,15 @@ class ProgramMakefile(Makefile):
     """
     def __init__(self, configuration, build_file=None, install_dir=None,
                  console=0, qt=0, opengl=0, python=0, threaded=0, warnings=1,
-                 debug=0, dir=None, makefile="Makefile", installs=None):
+                 debug=0, dir=None, makefile="Makefile", installs=None,
+                 universal=''):
         """Initialise an instance of a program Makefile.
 
         build_file is the file containing the target specific information.  If
         it is a dictionary instead then its contents are validated.
         install_dir is the directory the target will be installed in.
         """
-        Makefile.__init__(self, configuration, console, qt, opengl, python, threaded, warnings, debug, dir, makefile, installs)
+        Makefile.__init__(self, configuration, console, qt, opengl, python, threaded, warnings, debug, dir, makefile, installs, universal)
 
         self._install_dir = install_dir
 
@@ -1626,6 +1709,9 @@ class ProgramMakefile(Makefile):
             mfile.write("\t$(LINK) $(LFLAGS) /OUT:$(TARGET) @<<\n")
             mfile.write("\t  $(OFILES) $(LIBS)\n")
             mfile.write("<<\n")
+
+            if "embed_manifest_dll" in self.optional_list("CONFIG"):
+                mfile.write("\tmt -nologo -manifest $(TARGET).manifest -outputresource:$(TARGET);1\n")
         elif self.generator == "BMAKE":
             mfile.write("\t$(LINK) @&&|\n")
             mfile.write("\t$(LFLAGS) $(OFILES) ,$(TARGET),,$(LIBS),,\n")
@@ -1963,10 +2049,90 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
 
             orides[name] = prefix + oride[val_start:]
 
-    try:
-        f = open(filename, "r")
-    except IOError, detail:
-        error("Unable to open %s: %s" % (filename, detail))
+    # This class defines a file like object that handles the nested include()
+    # directives in qmake files.
+    class qmake_build_file_reader:
+        def __init__(self, filename):
+            self.filename = filename
+            self.currentfile = None
+            self.filestack = []
+            self.pathstack = []
+            self.cond_fname = None
+            self._openfile(filename)
+
+        def _openfile(self, filename):
+            try:
+                f = open(filename, 'r')
+            except IOError, detail:
+                # If this file is conditional then don't raise an error.
+                if self.cond_fname == filename:
+                    return
+
+                error("Unable to open %s: %s" % (filename, detail))
+
+            if self.currentfile:
+                self.filestack.append(self.currentfile)
+                self.pathstack.append(self.path)
+
+            self.currentfile = f
+            self.path = os.path.dirname(filename)
+
+        def readline(self):
+            line = self.currentfile.readline()
+            sline = line.strip()
+
+            if self.cond_fname and sline == '}':
+                # The current condition is closed.
+                self.cond_fname = None
+                line = self.currentfile.readline()
+            elif sline.startswith('exists(') and sline.endswith('{'):
+                # A new condition is opened so extract the filename.
+                self.cond_fname = self._normalise(sline[:-1].strip()[7:-1].strip())
+                line = self.currentfile.readline()
+            elif sline.startswith('include('):
+                nextfile = self._normalise(sline[8:-1].strip())
+                self._openfile(nextfile)
+                return self.readline()
+
+            if not line and self.filestack:
+                self.currentfile = self.filestack.pop()
+                self.path = self.pathstack.pop()
+                return self.readline()
+
+            return line
+
+        # Normalise a filename by expanding any environment variables and
+        # making sure it is absolute.
+        def _normalise(self, fname):
+            if "$(" in fname:
+                fname = os.path.normpath(self._expandvars(fname))
+
+            if not os.path.isabs(fname):
+                fname = os.path.join(self.path, fname)
+
+            return fname
+
+        # Expand the environment variables in a filename.
+        def _expandvars(self, fname):
+            i = 0
+            while True:
+                m = re.search(r'\$\((\w+)\)', fname[i:])
+                if not m:
+                    break
+
+                i, j = m.span(0)
+                name = m.group(1)
+                if name in os.environ:
+                    tail = fname[j:]
+                    fname = fname[:i] + os.environ[name]
+                    i = len(fname)
+                    fname += tail
+                else:
+                    i = j
+
+            return fname
+
+    f = qmake_build_file_reader(filename)
 
     # Get everything into a dictionary.
     raw = {
@@ -2064,7 +2230,10 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
                 try:
                     value = raw[lhs]
                 except KeyError:
-                    error("%s: macro '%s' is not defined." % (filename, lhs))
+                    # We used to treat this as an error, but Qt v4.3.0 has at
+                    # least one case that refers to an undefined macro.  If
+                    # qmake handles it then this must be the correct behaviour.
+                    value = ""
 
             rhs = rhs[:mstart] + value + rhs[mend:]
             estart = string.find(rhs, "$$(")
@@ -2172,7 +2341,16 @@ def create_wrapper(script, wrapper, gui=0):
 
         wf.write("@\"%s\" \"%s\" %%1 %%2 %%3 %%4 %%5 %%6 %%7 %%8 %%9\n" % (exe, script))
     elif sys.platform == "darwin":
-        wf.write("exec %sw %s ${1+\"$@\"}\n" % (sys.executable, script))
+        # python, pythonw and sys.executable are all different images.  We
+        # would prefer to use the latter (because it includes the version
+        # number) but that would mean being unable to support the "gui"
+        # argument.
+        if gui:
+            exe = "pythonw"
+        else:
+            exe = "python"
+
+        wf.write("exec %s %s ${1+\"$@\"}\n" % (exe, script))
     else:
         wf.write("exec %s %s ${1+\"$@\"}\n" % (sys.executable, script))
 
